@@ -49,9 +49,6 @@
 #include <src/udisksmodulemanager.h>
 
 #include "udiskslvm2daemonutil.h"
-#include "udiskslvm2dbusutil.h"
-#include "udiskslvm2state.h"
-#include "udisks-lvm2-generated.h"
 
 /**
  * SECTION:udiskslvm2daemonutil
@@ -249,25 +246,6 @@ udisks_daemon_util_lvm2_wipe_block (UDisksDaemon  *daemon,
 
 /* -------------------------------------------------------------------------------- */
 
-/*  transfer-none  */
-UDisksLinuxVolumeGroupObject *
-udisks_daemon_util_lvm2_find_volume_group_object (UDisksDaemon *daemon,
-                                                  const gchar  *name)
-{
-  UDisksLVM2State *state;
-  UDisksModuleManager *manager;
-
-  manager = udisks_daemon_get_module_manager (daemon);
-  g_assert (manager != NULL);
-
-  state = (UDisksLVM2State *) udisks_module_manager_get_module_state_pointer (manager, LVM2_MODULE_NAME);
-  g_assert (state != NULL);
-
-  return g_hash_table_lookup (udisks_lvm2_state_get_name_to_volume_group (state), name);
-}
-
-/* -------------------------------------------------------------------------------- */
-
 gboolean
 udisks_daemon_util_lvm2_name_is_reserved (const gchar *name)
 {
