@@ -24,6 +24,8 @@
 #include <gio/gio.h>
 #include <glib-unix.h>
 
+#include <blockdev/blockdev.h>
+
 #include "udiskslogging.h"
 #include "udisksdaemontypes.h"
 #include "udisksdaemon.h"
@@ -138,7 +140,7 @@ main (int    argc,
   if (opt_debug)
     {
       /* tell GLib logging to not throw away DEBUG and INFO messages (for our
-         "udisks" domain) unless already specied somehow by the user */
+         "udisks" domain) unless already specified somehow by the user */
       g_setenv("G_MESSAGES_DEBUG", "udisks", FALSE);
     }
 
@@ -147,6 +149,12 @@ main (int    argc,
 
   udisks_notice ("udisks daemon version %s starting", PACKAGE_VERSION);
 
+  if (! bd_utils_get_linux_version (&error))
+    {
+      udisks_warning ("Failed to retrieve kernel version: %s", error->message);
+      g_clear_error (&error);
+    }
+
   loop = g_main_loop_new (NULL, FALSE);
 
   if (!opt_no_sigint)
