@@ -3,25 +3,33 @@ import dbus
 import os
 from distutils.spawn import find_executable
 
+from config_h import UDISKS_MODULES_ENABLED
+
 class UdisksBaseTest(udiskstestcase.UdisksTestCase):
     '''This is a base test suite'''
 
-    udisks_modules = set(['Bcache', 'BTRFS', 'ISCSI.Initiator', 'LVM2', 'ZRAM', 'VDO'])
+    # A map between module name (ID) and corresponding org.freedesktop.UDisks2.Manager interface
+    UDISKS_MODULE_MANAGER_IFACES = {'bcache': 'Bcache',
+                                    'btrfs': 'BTRFS',
+                                    'iscsi': 'ISCSI.Initiator',
+                                    'lvm2': 'LVM2',
+                                    'zram': 'ZRAM',
+                                    'vdo': 'VDO'}
 
     def setUp(self):
         self.manager_obj = self.get_object('/Manager')
 
     def _get_modules(self):
-        distro, version = self.distro
-        modules = self.udisks_modules
-        if distro in ('enterprise_linux', 'centos') and version == '7':
-            modules = modules - {'Bcache'}
-        elif distro in ('enterprise_linux', 'centos') and int(version) > 7:
-            modules = modules - {'Bcache', 'BTRFS'}
-        # assuming the kvdo module is typically pulled in as a vdo tool dependency
-        if not find_executable("vdo"):
-            modules = modules - {'VDO'}
-        return modules
+        # The 'lsm' module is intentionally not tested here to avoid its initialization
+        # at this point. It's tested thoroughly in test_19_lsm.
+        return UDISKS_MODULES_ENABLED - {'lsm'}
+
+    def _get_udisks2_conf_path(self):
+        _CONF_FILE = 'udisks2.conf'
+        if os.environ['UDISKS_TESTS_ARG_SYSTEM'] == '1':
+            return os.path.join('/etc/udisks2/', _CONF_FILE)
+        else:
+            return os.path.join(os.environ['UDISKS_TESTS_PROJDIR'], 'udisks', _CONF_FILE)
 
     def test_10_manager(self):
         '''Testing the manager object presence'''
@@ -29,22 +37,46 @@ class UdisksBaseTest(udiskstestcase.UdisksTestCase):
         version = self.get_property(self.manager_obj, '.Manager', 'Version')
         version.assertIsNotNone()
 
+    def _restore_udisks2_conf(self):
+        if self.udisks2_conf_contents:
+            self.write_file(self._get_udisks2_conf_path(), self.udisks2_conf_contents)
+        else:
+            self.remove_file(self._get_udisks2_conf_path(), ignore_nonexistent=True)
+
     def test_20_enable_modules(self):
         manager = self.get_interface(self.manager_obj, '.Manager')
+
+        # make a backup of udisks2.conf
+        self.udisks2_conf_contents = None
+        try:
+            self.udisks2_conf_contents = self.read_file(self._get_udisks2_conf_path())
+        except FileNotFoundError as e:
+            # no existing udisks2.conf, simply remove the file once finished
+            pass
+
+        # mock the udisks2.conf to load only tested modules
+        contents = '[udisks2]\nmodules=%s\n' % ','.join(self._get_modules())
+        self.write_file(self._get_udisks2_conf_path(), contents)
+        self.addCleanup(self._restore_udisks2_conf)
+
+        manager.EnableModules(dbus.Boolean(True))
+
         manager_intro = dbus.Interface(self.manager_obj, "org.freedesktop.DBus.Introspectable")
         intro_data = manager_intro.Introspect()
-        modules = self._get_modules()
-        modules_loaded = any('interface name="%s.Manager.%s"' % (self.iface_prefix, module)
-                             in intro_data for module in modules)
+        for module in self._get_modules():
+            self.assertIn('interface name="%s.Manager.%s"' % (self.iface_prefix, self.UDISKS_MODULE_MANAGER_IFACES[module]), intro_data)
 
-        if modules_loaded:
-            self.skipTest("Modules already loaded, nothing to test")
-        else:
-            manager.EnableModules(dbus.Boolean(True))
-            intro_data = manager_intro.Introspect()
-
-            for module in modules:
-                self.assertIn('interface name="%s.Manager.%s"' % (self.iface_prefix, module), intro_data)
+    def test_21_enable_single_module(self):
+        manager = self.get_interface(self.manager_obj, '.Manager')
+        # Test that no error is returned for already loaded modules
+        for module in self._get_modules():
+            with self.assertRaises(dbus.exceptions.DBusException):
+                manager.EnableModule(module, dbus.Boolean(False))
+            manager.EnableModule(module, dbus.Boolean(True))
+        with self.assertRaises(dbus.exceptions.DBusException):
+            manager.EnableModule("nonexistent", dbus.Boolean(True))
+        with self.assertRaises(dbus.exceptions.DBusException):
+            manager.EnableModule("nonexistent", dbus.Boolean(False))
 
     def test_30_supported_filesystems(self):
         fss = self.get_property(self.manager_obj, '.Manager', 'SupportedFilesystems')
@@ -90,9 +122,9 @@ class UdisksBaseTest(udiskstestcase.UdisksTestCase):
         online_grow    = 0b10000
         manager = self.get_interface(self.manager_obj, '.Manager')
         with self.assertRaises(dbus.exceptions.DBusException):
-            manager.CanResize('nilfs2')
+            manager.CanResize('minix')
         avail, mode, util = manager.CanResize('xfs')
-        # the resize mode flage values are defined in the method documentation
+        # the resize mode flags values are defined in the method documentation
         self.assertEqual(mode, online_grow | offline_grow)
         if avail:
             self.assertEqual(util, '')
