@@ -4,7 +4,6 @@ import six
 import time
 
 import udiskstestcase
-from udiskstestcase import unstable_test
 
 BLOCK_SIZE = 512
 
@@ -175,8 +174,11 @@ class UdisksPartitionTableTest(udiskstestcase.UdisksTestCase):
         dbus_cont = self.get_property(log_part, '.Partition', 'IsContained')
         dbus_cont.assertTrue()
 
+        dbus_offset = self.get_property(log_part, '.Partition', 'Offset')
+        dbus_size = self.get_property(log_part, '.Partition', 'Size')
+
         # create one more logical partition
-        log_path2 = disk.CreatePartition(dbus.UInt64(51 * 1024**2), dbus.UInt64(50 * 1024**2), '', '',
+        log_path2 = disk.CreatePartition(dbus.UInt64(dbus_offset.value + dbus_size.value), dbus.UInt64(50 * 1024**2), '', '',
                                          log_options, dbus_interface=self.iface_prefix + '.PartitionTable')
         self.udev_settle()
 
@@ -615,7 +617,7 @@ class UdisksPartitionTest(udiskstestcase.UdisksTestCase):
         _ret, sys_type = self.run_command('blkid /dev/%s -p -o value -s PART_ENTRY_TYPE' % part_name)
         self.assertEqual(sys_type, part_type)
 
-    @unstable_test
+    @udiskstestcase.tag_test(udiskstestcase.TestTags.UNSTABLE)
     def test_resize(self):
         disk = self.get_object('/block_devices/' + os.path.basename(self.vdevs[0]))
         self.assertIsNotNone(disk)
