@@ -25,14 +25,16 @@ class UdisksVDOTest(udiskstestcase.UdisksTestCase):
     @classmethod
     def setUpClass(cls):
         udiskstestcase.UdisksTestCase.setUpClass()
-        if not cls.check_module_loaded('VDO'):
+        if not cls.check_module_loaded('vdo'):
             udiskstestcase.UdisksTestCase.tearDownClass()
             raise unittest.SkipTest('Udisks module for VDO tests not loaded, skipping.')
 
         if not find_executable('vdo'):
+            udiskstestcase.UdisksTestCase.tearDownClass()
             raise unittest.SkipTest('vdo executable not foundin $PATH, skipping.')
 
-        if not BlockDev.utils_have_kernel_module('kvdo'):
+        if not cls.module_available("kvdo"):
+            udiskstestcase.UdisksTestCase.tearDownClass()
             raise unittest.SkipTest('VDO kernel module not available, skipping.')
 
     def setUp(self):
@@ -71,6 +73,13 @@ class UdisksVDOTest(udiskstestcase.UdisksTestCase):
              if ret != 0:
                  self.fail('Failed to remove vdo volume %s: %s' % (vdo_name, out))
 
+    def test__manager_interface(self):
+        '''Test for module D-Bus Manager interface presence'''
+
+        manager = self.get_object('/Manager')
+        intro_data = manager.Introspect(self.no_options, dbus_interface='org.freedesktop.DBus.Introspectable')
+        self.assertIn('interface name="%s.Manager.VDO"' % self.iface_prefix, intro_data)
+
     def test_create_and_attributes(self):
         '''Test creating a new vdo volume and verify its properties'''
 
@@ -245,7 +254,7 @@ class UdisksVDOTest(udiskstestcase.UdisksTestCase):
         vdo.Remove(False, self.no_options, dbus_interface=self.iface_prefix + '.Block.VDO')
         self.assertFalse(os.path.exists('/dev/mapper/%s' % vdo_name))
 
-    @udiskstestcase.unstable_test
+    @udiskstestcase.tag_test(udiskstestcase.TestTags.UNSTABLE)
     def test_grow_physical(self):
         '''Test org.freedesktop.UDisks2.Block.VDO.GrowPhysical()'''
 
@@ -284,7 +293,8 @@ class UdisksVDOTest(udiskstestcase.UdisksTestCase):
         orig_physical_size.assertGreater(0)
 
         # no room to grow, expect a failure and no change in property value
-        msg = '(Cannot grow physical on VDO|Cannot prepare to grow physical on VDO .*; device-mapper: message ioctl on .* failed: Invalid argument)'
+        msg = '(Cannot grow physical on VDO|Cannot prepare to grow physical on VDO .*; device-mapper: message ioctl on .* failed: Invalid argument|'\
+              'Can\'t grow a VDO volume by less than 4096 bytes)'
         with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
             vdo.GrowPhysical(self.no_options, dbus_interface=self.iface_prefix + '.Block.VDO')
         new_physical_size = self.get_property(vdo, '.Block.VDO', 'PhysicalSize')
