@@ -41,9 +41,8 @@ void         udisks_linux_block_handle_format (UDisksBlock            *block,
                                                void                  (*complete)(gpointer user_data),
                                                gpointer                complete_user_data);
 
-gchar       *udisks_linux_get_parent_for_tracking (UDisksDaemon *daemon,
-                                                   const gchar    *path,
-                                                   const gchar   **uuid_ret);
+gboolean     udisks_linux_block_matches_id (UDisksLinuxBlock *block,
+                                            const gchar      *device_path);
 
 GVariant    *udisks_linux_find_child_configuration (UDisksDaemon *daemon,
                                                     const gchar    *uuid);
@@ -60,6 +59,8 @@ gboolean     udisks_linux_block_is_luks (UDisksBlock *block);
 
 gboolean     udisks_linux_block_is_tcrypt (UDisksBlock *block);
 
+gboolean     udisks_linux_block_is_bitlk (UDisksBlock *block);
+
 gboolean     udisks_linux_block_is_unknown_crypto (UDisksBlock *block);
 
 void         udisks_linux_block_encrypted_lock (UDisksBlock *block);
