@@ -95,8 +95,7 @@ gboolean tcrypt_open_job_func (UDisksThreadedJob  *job,
 {
   CryptoJobData *data = (CryptoJobData*) user_data;
 
-  // We always use the veracrypt option, because it can
-  // unlock both VeraCrypt and legacy TrueCrypt volumes
+  /* We always use the veracrypt option, because it can unlock both VeraCrypt and legacy TrueCrypt volumes */
   gboolean  veracrypt = TRUE;
 
   return bd_crypto_tc_open_full (data->device, data->map_name,
@@ -113,3 +112,23 @@ gboolean tcrypt_close_job_func (UDisksThreadedJob  *job,
   CryptoJobData *data = (CryptoJobData*) user_data;
   return bd_crypto_tc_close (data->map_name, error);
 }
+
+gboolean bitlk_open_job_func (UDisksThreadedJob  *job,
+                              GCancellable       *cancellable,
+                              gpointer            user_data,
+                              GError            **error)
+{
+  CryptoJobData *data = (CryptoJobData*) user_data;
+  return bd_crypto_bitlk_open (data->device, data->map_name,
+                               (const guint8*) data->passphrase->str, data->passphrase->len,
+                               data->read_only, error);
+}
+
+gboolean bitlk_close_job_func (UDisksThreadedJob  *job,
+                               GCancellable       *cancellable,
+                               gpointer            user_data,
+                               GError            **error)
+{
+  CryptoJobData *data = (CryptoJobData*) user_data;
+  return bd_crypto_bitlk_close (data->map_name, error);
+}
