@@ -146,6 +146,7 @@ udisks_linux_partition_table_update (UDisksLinuxPartitionTable  *table,
 
   udisks_partition_table_set_partitions (UDISKS_PARTITION_TABLE (table),
                                          partition_object_paths);
+  g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (table));
 
 
   g_free (partition_object_paths);
@@ -418,12 +419,12 @@ udisks_linux_partition_table_handle_create_partition (UDisksPartitionTable   *ta
   overlapping_part = bd_part_get_part_by_pos (device_name, offset, &error);
   if (overlapping_part != NULL && ! (overlapping_part->type & BD_PART_TYPE_FREESPACE))
     {
-      // extended partition or metadata of the extended partition
+      /* extended partition or metadata of the extended partition */
       if (overlapping_part->type & BD_PART_TYPE_EXTENDED || overlapping_part->type & (BD_PART_TYPE_LOGICAL | BD_PART_TYPE_METADATA))
         {
           if (overlapping_part->start == offset)
             {
-              // just add 1 byte, libblockdev will adjust it
+              /* just add 1 byte, libblockdev will adjust it */
               offset += 1;
               udisks_warning ("Requested start of the logical partition overlaps "
                               "with extended partition metadata. Start of the "
@@ -432,7 +433,7 @@ udisks_linux_partition_table_handle_create_partition (UDisksPartitionTable   *ta
         }
       else
         {
-          // overlapping partition is not a free space nor an extended part -> error
+          /* overlapping partition is not a free space nor an extended part -> error */
           g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
                                                  "Requested start for the new partition %"G_GUINT64_FORMAT" "
                                                  "overlaps with existing partition %s.",
@@ -488,6 +489,27 @@ udisks_linux_partition_table_handle_create_partition (UDisksPartitionTable   *ta
         }
     }
 
+  /* wipe the newly created partition if wanted */
+  if (part_spec->type != BD_PART_TYPE_EXTENDED)
+    {
+      if (!bd_fs_wipe (part_spec->path, TRUE, &error))
+        {
+          if (g_error_matches (error, BD_FS_ERROR, BD_FS_ERROR_NOFS))
+            g_clear_error (&error);
+          else
+            {
+              g_dbus_method_invocation_return_error (invocation,
+                                                     UDISKS_ERROR,
+                                                     UDISKS_ERROR_FAILED,
+                                                     "Error wiping newly created partition %s: %s",
+                                                     part_spec->path,
+                                                     error->message);
+              udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
+              goto out;
+            }
+        }
+    }
+
   wait_data->ignore_container = (part_spec->type == BD_PART_TYPE_LOGICAL);
   wait_data->pos_to_wait_for = part_spec->start + (part_spec->size / 2L);
 
@@ -499,7 +521,7 @@ udisks_linux_partition_table_handle_create_partition (UDisksPartitionTable   *ta
                                                          wait_for_partition,
                                                          wait_data,
                                                          NULL,
-                                                         30,
+                                                         UDISKS_DEFAULT_WAIT_TIMEOUT,
                                                          &error);
   if (partition_object == NULL)
     {
@@ -518,33 +540,12 @@ udisks_linux_partition_table_handle_create_partition (UDisksPartitionTable   *ta
       goto out;
     }
 
-  /* wipe the newly created partition if wanted */
-  if (part_spec->type != BD_PART_TYPE_EXTENDED)
-    {
-      if (!bd_fs_wipe (part_spec->path, TRUE, &error))
-        {
-          if (g_error_matches (error, BD_FS_ERROR, BD_FS_ERROR_NOFS))
-            g_clear_error (&error);
-          else
-            {
-              g_dbus_method_invocation_return_error (invocation,
-                                                     UDISKS_ERROR,
-                                                     UDISKS_ERROR_FAILED,
-                                                     "Error wiping newly created partition %s: %s",
-                                                     udisks_block_get_device (partition_block),
-                                                     error->message);
-              udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
-              g_clear_object (&partition_object);
-              goto out;
-            }
-        }
-    }
-
   /* Trigger uevent on the disk -- we sometimes get add-remove-add uevents for
      the new partition without getting change event for the disks after the
      last add event and this breaks the "Partitions" property on the
      "PartitionTable" interface. */
-  udisks_linux_block_object_trigger_uevent (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_linux_block_object_trigger_uevent_sync (UDISKS_LINUX_BLOCK_OBJECT (object),
+                                                 UDISKS_DEFAULT_WAIT_TIMEOUT);
 
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
